#include <string.h>
#include "emu.h"
#include "dip.h"

WINDOW *dip_switch_window=NULL;
byte dip_sw_1, dip_sw_2;

int Emu_DipSwitches(WINDOW *window, GADGET *d)
{
   int x, y;

   if (dip_switch_window)
   {
      GUI_SwitchFocus(dip_switch_window);
      return D_O_K;
   }

   /* create the DIP switch window */
   GUI_PrintLog(FALSE, "Creating DIP-switch window.");

   x = get_config_int("DIPswitch_window", "x_position", 120);
   y = get_config_int("DIPswitch_window", "y_position", 110);
   dip_switch_window = GUI_NewWindow(x, y, gui_dipswitch_dialog, &DIP_Shutdown);
   if (!dip_switch_window)
      return D_O_K;

   return D_O_K;
}

void DIP_LoadSwitches(void)
{
   MSG msg;
   GADGET *gadget;

   GUI_PrintLog(FALSE, "Loading DIP switch settings.");
   dip_sw_1 = get_config_int(current_game->short_name, "dip_sw_1", 0x00);
   dip_sw_2 = get_config_int(current_game->short_name, "dip_sw_2", 0x00);

   /* clear current DIP settings */
   gadget = &gui_dipswitch_dialog[0];
   while(gadget->proc)
   {
      gadget->flags &= ~D_SELECTED;
      gadget++;
   }

   if (dip_sw_1 & 1) /* Game style */
   {
      gui_dipswitch_dialog[14].flags |= D_SELECTED;
   } else {
      gui_dipswitch_dialog[15].flags |= D_SELECTED;
   }

   if (dip_sw_1 & 2) /* Screen flip */
   {
      gui_dipswitch_dialog[11].flags |= D_SELECTED;
   } else {
      gui_dipswitch_dialog[12].flags |= D_SELECTED;
   }

   if (dip_sw_1 & 4) /* Test mode */
   {
      gui_dipswitch_dialog[8].flags |= D_SELECTED;
   } else {
      gui_dipswitch_dialog[9].flags |= D_SELECTED;
   }

   switch (dip_sw_1 & 48) /* Coin price A */
   {
   case 0: gui_dipswitch_dialog[17].flags |= D_SELECTED; break;
   case 1: gui_dipswitch_dialog[18].flags |= D_SELECTED; break;
   case 2: gui_dipswitch_dialog[19].flags |= D_SELECTED; break;
   case 3: gui_dipswitch_dialog[20].flags |= D_SELECTED; break;
   }

   switch (dip_sw_1 & 192) /* Coin price B */
   {
   case 0: gui_dipswitch_dialog[22].flags |= D_SELECTED; break;
   case 1: gui_dipswitch_dialog[23].flags |= D_SELECTED; break;
   case 2: gui_dipswitch_dialog[24].flags |= D_SELECTED; break;
   case 3: gui_dipswitch_dialog[25].flags |= D_SELECTED; break;
   }

   switch (dip_sw_2 & 3) /* Difficulty */
   {
   case 0: gui_dipswitch_dialog[27].flags |= D_SELECTED; break;
   case 1: gui_dipswitch_dialog[28].flags |= D_SELECTED; break;
   case 2: gui_dipswitch_dialog[29].flags |= D_SELECTED; break;
   case 3: gui_dipswitch_dialog[30].flags |= D_SELECTED; break;
   }

   switch (dip_sw_2 & 12) /* Bonus points */
   {
   case 0: gui_dipswitch_dialog[32].flags |= D_SELECTED; break;
   case 1: gui_dipswitch_dialog[33].flags |= D_SELECTED; break;
   case 2: gui_dipswitch_dialog[34].flags |= D_SELECTED; break;
   case 3: gui_dipswitch_dialog[35].flags |= D_SELECTED; break;
   }

   switch (dip_sw_2 & 48) /* Number of lives */
   {
   case 0: gui_dipswitch_dialog[37].flags |= D_SELECTED; break;
   case 1: gui_dipswitch_dialog[38].flags |= D_SELECTED; break;
   case 2: gui_dipswitch_dialog[39].flags |= D_SELECTED; break;
   case 3: gui_dipswitch_dialog[40].flags |= D_SELECTED; break;
   }

   if (dip_sw_2 & 64) /* Continues */
   {
      gui_dipswitch_dialog[43].flags |= D_SELECTED;
   } else {
      gui_dipswitch_dialog[42].flags |= D_SELECTED;
   }

   gui_dipswitch_dialog[2].d1 = dip_sw_1;
   gui_dipswitch_dialog[3].d1 = dip_sw_2;

   if (stricmp(current_game->short_name, "tnzs_alt") == 0)
   {
      /* set bonus points text labels */
      gui_dipswitch_dialog[32].dp = tnzs_alt_bonus_settings[0];
      gui_dipswitch_dialog[33].dp = tnzs_alt_bonus_settings[1];
      gui_dipswitch_dialog[34].dp = tnzs_alt_bonus_settings[2];
      gui_dipswitch_dialog[35].dp = tnzs_alt_bonus_settings[3];
   } else {
      /* set bonus points text labels */
      gui_dipswitch_dialog[32].dp = tnzs_bonus_settings[0];
      gui_dipswitch_dialog[33].dp = tnzs_bonus_settings[1];
      gui_dipswitch_dialog[34].dp = tnzs_bonus_settings[2];
      gui_dipswitch_dialog[35].dp = tnzs_bonus_settings[3];
   }
   if (dip_switch_window)
   {
      msg.type = MSG_DRAW;
      GUI_PostMessage(&dip_switch_window->msg_queue, &msg);
      msg.type = MSG_REFRESH;
      GUI_PostMessage(&dip_switch_window->msg_queue, &msg);
   }
}

void DIP_Shutdown(WINDOW *window)
{
   GUI_PrintLog(FALSE, "Saving DIP switch settings.");
   set_config_int(current_game->short_name, "dip_sw_1", dip_sw_1);
   set_config_int(current_game->short_name, "dip_sw_2", dip_sw_2);

   if (window)
   {
      set_config_int("DIPswitch_window", "x_position", window->x);
      set_config_int("DIPswitch_window", "y_position", window->y);
   }
   dip_switch_window = NULL;
}

int gui_dipbox_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int dip, x;
   byte value;
   switch(msg->type)
   {
   case MSG_DRAW:
      value = d->d1;
      GUI_DrawRect3D(FALSE, d->bmp, d->w - 1, d->h - 1, 0, 8);
      text_mode(GUI_BG_NORMAL);
      for (dip=0,x=1;dip<8;dip++)
      {
         if (value & 0x80)
         {
            blit(gui_data[GUI_SWITCH1].dat, d->bmp, 0, 0, x, 9, 8, 16);
         } else {
            blit(gui_data[GUI_SWITCH2].dat, d->bmp, 0, 0, x, 9, 8, 16);
         }
         textprintf(d->bmp, bios_font, x, 0, GUI_BLACK, "%i", 8-dip);
         x += 8;
         value <<= 1;
      }
      return D_O_K;
   }
   return D_O_K;
}

int DIP_UpdateSettings(WINDOW *window, GADGET *d)
{
   if (!current_game) return D_O_K;

   dip_sw_1 = dip_sw_2 = 0x00;
   if (gui_dipswitch_dialog[8].flags & D_SELECTED) dip_sw_1 |= 4;
   if (gui_dipswitch_dialog[11].flags & D_SELECTED) dip_sw_1 |= 2;
   if (gui_dipswitch_dialog[14].flags & D_SELECTED) dip_sw_1 |= 1;

   if (gui_dipswitch_dialog[18].flags & D_SELECTED) dip_sw_1 |= 16;
   if (gui_dipswitch_dialog[19].flags & D_SELECTED) dip_sw_1 |= 32;
   if (gui_dipswitch_dialog[20].flags & D_SELECTED) dip_sw_1 |= 48;

   if (gui_dipswitch_dialog[23].flags & D_SELECTED) dip_sw_1 |= 64;
   if (gui_dipswitch_dialog[24].flags & D_SELECTED) dip_sw_1 |= 128;
   if (gui_dipswitch_dialog[25].flags & D_SELECTED) dip_sw_1 |= 192;

   if (gui_dipswitch_dialog[28].flags & D_SELECTED) dip_sw_2 |= 1;
   if (gui_dipswitch_dialog[29].flags & D_SELECTED) dip_sw_2 |= 2;
   if (gui_dipswitch_dialog[30].flags & D_SELECTED) dip_sw_2 |= 3;

   if (gui_dipswitch_dialog[33].flags & D_SELECTED) dip_sw_2 |= 4;
   if (gui_dipswitch_dialog[34].flags & D_SELECTED) dip_sw_2 |= 8;
   if (gui_dipswitch_dialog[35].flags & D_SELECTED) dip_sw_2 |= 12;

   if (gui_dipswitch_dialog[38].flags & D_SELECTED) dip_sw_2 |= 16;
   if (gui_dipswitch_dialog[39].flags & D_SELECTED) dip_sw_2 |= 32;
   if (gui_dipswitch_dialog[40].flags & D_SELECTED) dip_sw_2 |= 48;

   if (gui_dipswitch_dialog[43].flags & D_SELECTED) dip_sw_2 |= 64;

   gui_dipswitch_dialog[2].d1 = dip_sw_1;
   gui_dipswitch_dialog[3].d1 = dip_sw_2;
   gui_dipswitch_dialog[2].flags |= D_DIRTY;
   gui_dipswitch_dialog[3].flags |= D_DIRTY;

   return D_O_K;
}

char *tnzs_bonus_settings[4] = {
   "70,000 / 200,000",
   "200,000 / 300,000",
   "100,000 / 250,000",
   "50,000 / 150,000"
};   

char *tnzs_alt_bonus_settings[4] = {
   "10,000 / 150,000",
   "10,000 / 200,000",
   "10,000 / 300,000",
   "10,000 / 100,000"
};   

GADGET gui_dipswitch_dialog[] = {
   /* (dialog proc)       (x)          (y)       (w)       (h)       (flags)  (d1) (d2)  (dp) */
   { gui_window_proc,     0,           0,        NC_W+364, NC_H+243, 0,       0,   0,    "DIP Switches" },
   { gui_closebox_proc,   NC_W+364-12, 2,        9,        9,        D_EXIT,  0,   0,    NULL },

   /* no. 2 - DIP-boxes */
   { gui_dipbox_proc,     NC_X+65,     NC_Y+5,   64+2,     24+2,     0,       0,   0,    NULL },
   { gui_dipbox_proc,     NC_X+65,     NC_Y+36,  64+2,     24+2,     0,       0,   0,    NULL },
   { gui_text_proc,       NC_X+5,      NC_Y+13,  60,       16+2,     0,       0,   0,    "DIPSW A:" },
   { gui_text_proc,       NC_X+5,      NC_Y+44,  60,       16+2,     0,       0,   0,    "DIPSW B:" },
   { gui_line_proc,       NC_X,        NC_Y+67,  364,      2,        0,       0,   0,    NULL },

   /* no. 7 - Test mode */
   { gui_frame_proc,      NC_X+5,      NC_Y+76,  70,       52,       0,       0,   0,    "Test mode:" },
   { gui_radio_proc,      NC_X+10,     NC_Y+90,  60,       16,       0,       1,   0,    "On", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+10,     NC_Y+106, 60,       16,       0,       1,   0,    "Off", &DIP_UpdateSettings },

   /* no. 10 - Screen flip */
   { gui_frame_proc,      NC_X+5,      NC_Y+131, 70,       52,       0,       0,   0,    "Screen flip:" },
   { gui_radio_proc,      NC_X+10,     NC_Y+145, 60,       16,       0,       2,   0,    "Flipped", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+10,     NC_Y+161, 60,       16,       0,       2,   0,    "Normal", &DIP_UpdateSettings },

   /* no. 13 - Game style */
   { gui_frame_proc,      NC_X+5,      NC_Y+186, 70,       52,       0,       0,   0,    "Game style:" },
   { gui_radio_proc,      NC_X+10,     NC_Y+200, 60,       16,       0,       3,   0,    "Upright", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+10,     NC_Y+216, 60,       16,       0,       3,   0,    "Table", &DIP_UpdateSettings },

   /* no. 16 - Coin A pricing */
   { gui_frame_proc,      NC_X+80,     NC_Y+76,  90,       80,       0,       0,   0,    "Coin pricing (A):" },
   { gui_radio_proc,      NC_X+85,     NC_Y+90,  80,       16,       0,       4,   0,    "1 coin=1 play", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+85,     NC_Y+106, 80,       16,       0,       4,   0,    "2 coins=1 play", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+85,     NC_Y+122, 80,       16,       0,       4,   0,    "3 coins=1 play", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+85,     NC_Y+138, 80,       16,       0,       4,   0,    "4 coins=1 play", &DIP_UpdateSettings },

   /* no. 21 - Coin B pricing */
   { gui_frame_proc,      NC_X+80,     NC_Y+158, 90,       80,       0,       0,   0,    "Coin pricing (B):" },
   { gui_radio_proc,      NC_X+85,     NC_Y+172, 80,       16,       0,       5,   0,    "1 coin=2 plays", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+85,     NC_Y+188, 80,       16,       0,       5,   0,    "1 coin=3 plays", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+85,     NC_Y+204, 80,       16,       0,       5,   0,    "1 coin=4 plays", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+85,     NC_Y+220, 80,       16,       0,       5,   0,    "1 coin=6 plays", &DIP_UpdateSettings },

   /* no. 26 - Difficulty */
   { gui_frame_proc,      NC_X+175,    NC_Y+76,  90,       80,       0,       0,   0,    "Difficulty:" },
   { gui_radio_proc,      NC_X+180,    NC_Y+90,  80,       16,       0,       6,   0,    "Easy", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+180,    NC_Y+106, 80,       16,       0,       6,   0,    "Medium", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+180,    NC_Y+122, 80,       16,       0,       6,   0,    "Hard", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+180,    NC_Y+138, 80,       16,       0,       6,   0,    "Hardest", &DIP_UpdateSettings },

   /* no. 31 - Bonus points */
   { gui_frame_proc,      NC_X+175,    NC_Y+158, 110,      80,       0,       0,   0,    "Bonus points at:" },
   { gui_radio_proc,      NC_X+180,    NC_Y+172, 100,      16,       0,       7,   0,    NULL, &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+180,    NC_Y+188, 100,      16,       0,       7,   0,    NULL, &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+180,    NC_Y+204, 100,      16,       0,       7,   0,    NULL, &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+180,    NC_Y+220, 100,      16,       0,       7,   0,    NULL, &DIP_UpdateSettings },

   /* no. 36 - Number of lives */
   { gui_frame_proc,      NC_X+270,    NC_Y+76,  90,       80,       0,       0,   0,    "Number of lives:" },
   { gui_radio_proc,      NC_X+275,    NC_Y+90,  80,       16,       0,       8,   0,    "3 lives", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+275,    NC_Y+106, 80,       16,       0,       8,   0,    "2 lives", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+275,    NC_Y+122, 80,       16,       0,       8,   0,    "5 lives", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+275,    NC_Y+138, 80,       16,       0,       8,   0,    "4 lives", &DIP_UpdateSettings },

   /* no. 41 - Continues */
   { gui_frame_proc,      NC_X+290,    NC_Y+158, 70,       54,       0,       0,   0,    "Continues:" },
   { gui_radio_proc,      NC_X+295,    NC_Y+172, 60,       16,       0,       9,   0,    "On", &DIP_UpdateSettings },
   { gui_radio_proc,      NC_X+295,    NC_Y+188, 60,       16,       0,       9,   0,    "Off", &DIP_UpdateSettings },

   /* OK button */
   { gui_button_proc,     NC_X+290,    NC_Y+218, 70,       20,       D_EXIT,  0,   0,    "OK", NULL },
   { gui_text_proc,       NC_X+140,    NC_Y+15,  240,      16,       0,       0,   0,    "NOTE: You may have to reset the emulation" },
   { gui_text_proc,       NC_X+180,    NC_Y+31,  240,      16,       0,       0,   0,    "for the new settings to take effect." },
   { NULL }
};

/* from Greg Best <gregbest98@hotmail.com>:

<> SETTING OF DIP SWITCH A    (*):FACTORY SETTINGS
SETTING | POSITION | SWITCH 
GAME STYLE | TABLE | 1 OFF
           UPRIGHT | 1 ON
SCREEN INVERSION | * NORMAL | 2 OFF
                  INVERSION | 2 ON
TEST MODE | * NORMAL GAME | 3 OFF
                TEST MODE | 3 ON
SWITCH 4 ALWAYS OFF
PLAY PRICING COIN A  | * 1 COIN 1 PLAY | 5 OFF 6 OFF
                        2 COINS 1 PLAY | 5 ON  6 OFF
                        3 COINS 1 PLAY | 5 OFF 6 ON
                        4 COINS 1 PLAY | 5 ON  6 ON
PLAY PRICING COIN B  | * 1 COIN 2 PLAYS| 7 OFF 8 OFF
                         1 COIN 3 PLAYS| 7 ON  8 OFF
                         1 COIN 4 PLAYS| 7 OFF 8 ON
                         1 COIN 6 PLAYS| 7 ON  8 ON

<> SETTING OF DIP SWITCH B
SETTING | POSITION | SWITCH
GAME DIFFICULTY (A)EASY - (D)HARD | * A | 1 OFF 2 OFF
                                      B | 1 ON  2 OFF
                                      C | 1 OFF 2 ON
                                      D | 1 ON  2 ON
BONUS POINTS 1ST/2ND(EVERY) | * 70,000 / 200,000 PTS | 3 OFF 4 OFF
                               200,000 / 300,000 PTS | 3 ON  4 OFF
                               100,000 / 250,000 PTS | 3 OFF 4 ON
                                50,000 / 150,000 PTS | 3 ON  4 ON
NUMBER OF PLAYER | * 3 | 5 OFF 6 OFF
                     2 | 5 ON  6 OFF
                     5 | 5 OFF 6 ON
                     4 | 5 ON  6 ON
CONTINUED PLAY | * WITH | 7 OFF
                WITHOUT | 7 ON
SWITCH 8 ALWAYS OFF */

