#include "emu.h"
#include "utils.h"

int palette_map[512];
int num_colours, emu_palette_dirty;

void Palette_Recalc(byte *mem)
{
   int c,r,g,b,n;
   RGB rgb;

   num_colours = GUI_USER_COLOURS;
   mem += 511*2;
   for (n=511;n>=0;n--,mem-=2)
   {
      if ((n&15) == 0)
      {
      /* colour 0 for each 4-plane gfx tile is transparent */
         palette_map[n] = GUI_TRANSPARENT;
         continue;
      }

      /* get colour information */
      c = (mem[0]) | (mem[1] << 8);

      r = (c >> 10) & 31;
      g = (c >> 5) & 31;
      b = c & 31;

      /* add new colour, merging if necessary */
      c = Palette_FindColour(r << 3, g << 3, b << 3);

      if (c != -1)
      {
         palette_map[n] = c;
      } else {
         if (num_colours < 256)
         {
            palette_map[n] = num_colours;
            rgb.r = r << 1;
            rgb.g = g << 1;
            rgb.b = b << 1;
            gui_palette[num_colours] = rgb;
            emu_palette_dirty = TRUE;
            num_colours++;
         } else {
            palette_map[n] = bestfit_color(gui_palette, r << 1, g << 1, b << 1);
         }
      }
   }

   Emu_RequestRedraw();
   PalView_Update();
}

int Palette_FindColour(int r, int g, int b)
{
   int c;

   r >>= 2;
   g >>= 2;
   b >>= 2;

   for (c=1;c<num_colours;c++)
   {
      if (gui_palette[c].r == r &&
          gui_palette[c].g == g &&
          gui_palette[c].b == b) return c;
   }
   return -1;
}
