#include "gui.h"

int gui_holding_mouse;
int gfx_on=FALSE, gui_display_log_on_exit=FALSE;
int gui_mouse_focus_x, gui_mouse_focus_y;
BITMAP *gui_mouse_bitmap, *gui_backdrop;
MSG_QUEUE global_msg_queue;
DATAFILE *gui_data=NULL;

int GUI_Run(void)
{
   MSG msg;
   int result;

   show_mouse(screen);
   while(1)
   {
      if (!mouse_b) gui_holding_mouse = FALSE;

      if (mouse_b && !gui_holding_mouse)
      {
         /* dispatch mouse-click message */
         gui_holding_mouse = TRUE;
         msg.type = MSG_CLICK;
         msg.x = mouse_x;
         msg.y = mouse_y;

         GUI_PostMessage(&global_msg_queue, &msg);
      }
      if (keypressed())
      {
         /* dispatch keyboard message */
         msg.type = MSG_CHAR;
         msg.key = readkey();

         GUI_PostMessage(&global_msg_queue, &msg);
      }

      /* post an IDLE message */
      msg.type = MSG_IDLE;
      GUI_PostMessage(&global_msg_queue, &msg);

      while (GUI_GetMessage(&global_msg_queue, &msg))
      {
         /* process waiting messages */
         if (msg.type == MSG_END)
         {
            fade_out_range(8, GUI_BACKDROP_START, GUI_BACKDROP_END);
            fade_out(10);
            vsync();
            clear(screen);
            result = GUI_DispatchGlobalMessage(&msg);
            vsync();
            GUI_Shutdown();
            return TRUE;
         } else {
            result = GUI_DispatchGlobalMessage(&msg);
            if (result & D_CLOSE)
            {
               /* post quit message */
               msg.type = MSG_END;
               GUI_PostMessage(&global_msg_queue, &msg);
            }
         }
      }
   }
}

void GUI_PostMessage(MSG_QUEUE *queue, MSG *msg)
{
   queue->msg[queue->tail] = *msg;
   queue->tail++;
   if (queue->tail >= MAX_MESSAGES) queue->tail = 0;
   if (queue->tail == queue->head)
   {
      queue->head++;
      if (queue->head >= MAX_MESSAGES) queue->head = 0;
   }
}

int GUI_GetMessage(MSG_QUEUE *queue, MSG *msg)
{
   if (queue->head == queue->tail) return FALSE;

   *msg = queue->msg[queue->head];
   queue->head++;
   if (queue->head >= MAX_MESSAGES) queue->head = 0;
   return TRUE;
}

int GUI_Init(char *datafile)
{
   MSG msg;
   int x,y,gfx_w, gfx_h;
   GUI_PrintLog(FALSE, "Initialising GUI...");

   gui_data = load_datafile(datafile);
   if (!gui_data)
   {
      GUI_PrintLog(TRUE, "Cannot load %s", datafile);
      GUI_Shutdown();
      return FALSE;
   }

   GUI_PrintLog(FALSE, "Installing timer handler.");
   if (install_timer() == -1)
   {
      GUI_PrintLog(TRUE, "Cannot install timer handler!");
      GUI_Shutdown();
      return FALSE;
   }

   GUI_PrintLog(FALSE, "Installing mouse handler.");
   if (install_mouse() == -1)
   {
      GUI_PrintLog(TRUE, "Cannot find mouse.");
      GUI_PrintLog(TRUE, "Have you installed your mouse driver?");
      GUI_Shutdown();
      return FALSE;
   }

   GUI_PrintLog(FALSE, "Installing keyboard handler.");
   if (install_keyboard() == -1)
   {
      GUI_PrintLog(TRUE, "Cannot install keyboard handler!");
      GUI_Shutdown();
      return FALSE;
   }

   //gfx_w = get_config_int("GUI_settings", "x_resolution", 640);
   //gfx_h = get_config_int("GUI_settings", "y_resolution", 480);
   gfx_w = 640;
   gfx_h = 480;

   GUI_PrintLog(FALSE, "Setting graphics mode (%ix%i, 256 colours).", gfx_w, gfx_h);
   if (set_gfx_mode(GFX_AUTODETECT, gfx_w, gfx_h, 0,0) < 0)
   {
      GUI_PrintLog(TRUE, "Cannot set graphics mode (%ix%i, 256 colours).", gfx_w, gfx_h);
      GUI_PrintLog(TRUE, " ");
      GUI_PrintLog(TRUE, "You might need to load a VESA driver first.");
      GUI_PrintLog(TRUE, "If you do not have a VESA driver, try UniVBE (http://www.scitechsoft.com/).");
      GUI_PrintLog(TRUE, "You could also try changing the settings in %s to use a different mode.", datafile);
      return FALSE;
   }

   gfx_on = TRUE;
   gui_display_log_on_exit = TRUE;
   font = gui_data[GUI_FONT].dat;
   gui_backdrop = gui_data[GUI_BACKGROUND].dat;
   gui_font_baseline = 2;
   GUI_SetMouseCursor(gui_data[GUI_BUSY].dat, 1, 1);

   /* re-base backdrop */
   for (y=0;y<gui_backdrop->h;y++)
   {
      for (x=0;x<gui_backdrop->w;x++)
      {
         gui_backdrop->line[y][x] += GUI_BACKDROP_START-1;
      }
   }

   GUI_PrintLog(FALSE, "Installing sound drivers.");
   if (install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL) == -1)
   {
      GUI_PrintLog(TRUE, "Cannot install sound hardware.");
   }

   GUI_PrintLog(FALSE, "Installing joystick drivers.");
   if (install_joystick(JOY_TYPE_AUTODETECT) == -1)
   {
      GUI_PrintLog(TRUE, "No joystick found.");
   }

   global_msg_queue.head = global_msg_queue.tail = 0;
   msg.type = MSG_START;
   GUI_PostMessage(&global_msg_queue, &msg);

   GUI_SetMouseCursor(gui_data[GUI_CURSOR].dat, 1, 1);
   GUI_PrintLog(FALSE, "GUI subsystem initialised.");
   gui_display_log_on_exit = FALSE;
   return TRUE;
}

void GUI_Shutdown(void)
{
   GUI_PrintLog(FALSE, "Shutting down GUI.");
   //set_config_int("GUI_settings", "x_resolution", SCREEN_W);
   //set_config_int("GUI_settings", "y_resolution", SCREEN_H);

   allegro_exit();

   if (!gfx_on && gui_display_log_on_exit)
   {
      printf("\n");
      while (log_head != log_tail)
      {
         printf("%s\n", log_text[log_head]);
         log_head++;
         if (log_head >= LOG_LINES) log_head = 0;
      }
   }
   gfx_on = FALSE;
}

void GUI_SetMouseCursor(BITMAP *bmp, int focus_x, int focus_y)
{
   gui_mouse_bitmap = bmp;
   gui_mouse_focus_x = focus_x;
   gui_mouse_focus_y = focus_y;

   scare_mouse();
   set_mouse_sprite(bmp);
   set_mouse_sprite_focus(focus_x, focus_y);
   unscare_mouse();
}
