#include <stdarg.h>
#include "gui.h"

FILE *log_file=NULL;
int log_updated, log_head=0, log_tail=0;
char log_text[LOG_LINES][300];
int log_urgent[LOG_LINES];

int GUI_InitLog(void)
{
   int x, y;
   GUI_PrintLog(FALSE, "Initalising log window.");
   x = get_config_int("Log_window", "x_position", 180);
   y = get_config_int("Log_window", "y_position", 300);
   if (!GUI_NewWindow(x, y, gui_log_dialog, &GUI_CloseLog)) return FALSE;
   log_updated = TRUE;
   GUI_PrintLog(FALSE, "Log window startup OK.");
   return TRUE;
}

void GUI_CloseLog(WINDOW *window)
{
   set_config_int("Log_window", "x_position", window->x);
   set_config_int("Log_window", "y_position", window->y);
}

void GUI_PrintLog(int urgent, char *msg, ...)
{
   char text[300];
   va_list ap;
  
   va_start(ap, msg);
   vsprintf(text, msg, ap);
   va_end(ap);

   strcpy(log_text[log_tail], text);
   log_urgent[log_tail] = urgent;

   log_tail++;
   if (log_tail >= LOG_LINES) log_tail = 0;
   if (log_tail == log_head)
   {
      log_head++;
      if (log_head >= LOG_LINES) log_head = 0;
   }

   if (log_file) fprintf(log_file, "%s\n", text);
   fflush(log_file);
   if (!gfx_on) printf("%s\n", text);
   log_updated = TRUE;
}

GADGET gui_log_dialog[] = {
   /* (dialog proc)   (x)    (y)   (w)       (h)           (flags)  (d1) (d2)  (dp) */
   { gui_window_proc, 0,     0,    NC_W+400, NC_H+10*12+2, 0,       0,   0,    "Message log" },
   { gui_log_proc,    NC_X,  NC_Y, 400,      10*12+2,      0,       0,   0,    NULL,         },
   { NULL }
};

int gui_log_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int n, start, end, y;
   switch(msg->type)
   {
   case MSG_IDLE:
      if (log_updated)
      {
         d->flags = D_DIRTY;
      }
      return D_O_K;
   case MSG_DRAW:
      rectfill(d->bmp, 0, 0, d->w-1, d->h-1, GUI_WHITE);

      text_mode(GUI_WHITE);
      start = log_head;
      end = log_tail;
      n = 0;
      while (start != end)
      {
         y = 1 + n*text_height(font);
         if (log_urgent[start])
         {
            textout(d->bmp, font, log_text[start],
                    1, y, GUI_URGENT_TEXT);
            hline(d->bmp, 1, y + text_height(font) - gui_font_baseline,
                  text_length(font, log_text[start]), GUI_URGENT_TEXT);
         } else {
            textout(d->bmp, font, log_text[start],
                    1, y, GUI_BLACK);
         }
         n++;
         start++;
         if (start >= LOG_LINES) start = 0;
      }
      log_updated = FALSE;
      return D_O_K;
   }
   return D_O_K;
}
