#include "raze.h"

typedef unsigned char byte;
typedef unsigned short word;
typedef unsigned long dword;

#define Z80_IGNORE_INT -1
int Z80_Interrupt(void);

#define MAX_CPUS 2
#define MAX_AUDIO_CHIPS 1

typedef struct {
   int hz;

   byte *image;
   int icount, iperiod;
   void *regs;
} CPU;

typedef struct {
   char *description;
   int (*Init)(int num, int clock, int sample_rate, int buffer_size);
   void (*Reset)(int num);
   void (*Emulate)(int num);
   void (*Shutdown)(void);
} AUDIO_CHIP_DRIVER;

typedef struct {
   AUDIO_CHIP_DRIVER *driver;
   int num, hz;
} AUDIO_CHIP;

typedef struct {
   char *filename;
   int size;
   unsigned long checksum;
   byte *rom;
} ROM_LIST;

typedef struct {
   int interrupt_rate, gfx_w, gfx_h, interleave;

   int (*Init)(void);
   void (*Reset)(void);
   void (*Shutdown)(void);
   void (*DrawScreen)(BITMAP *bmp);

   int num_cpus;
   CPU cpu[MAX_CPUS];
   int num_audio_chip_types;
   AUDIO_CHIP audio_chip_types[MAX_AUDIO_CHIPS];
} MACHINE;

typedef struct {
   char *long_name, *short_name;
   MACHINE *machine;
   ROM_LIST *rom_list;
} GAME;

extern byte sound_latch;
extern int current_cpu;
