#ifndef __RM_GUI_INCLUDED__
#define __RM_GUI_INCLUDED__

#include <stdio.h>
#include <allegro.h>
#include "data.h"

/* general #define's */

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

/* message-box constants */
#define MB_OK     1   
#define MB_YESNO  2

/* how many lines should the log buffer store (add 1 extra) */
#define LOG_LINES 11

/* GUI colours */
#define GUI_TRANSPARENT    0
#define GUI_BLACK          1
#define GUI_WHITE          2
#define GUI_URGENT_TEXT    3
#define GUI_DISABLED_TEXT  4
#define GUI_BACKDROP       5
#define GUI_FG_TEXT        6
#define GUI_FG_HIGHLIGHT   7
#define GUI_FG_NORMAL      8
#define GUI_FG_SHADOW      9
#define GUI_BG_TEXT        10
#define GUI_BG_HIGHLIGHT   11
#define GUI_BG_NORMAL      12
#define GUI_BG_SHADOW      13
#define GUI_BACKDROP_START 14
#define GUI_BACKDROP_END   28
#define GUI_USER_COLOURS   29

/* how many messages per queue */
#define MAX_MESSAGES 256

/* size of the non-client area of a window */
#define NC_X 4
#define NC_Y 18
#define NC_W 8
#define NC_H 22

/* window flags */
#define W_VSYNC      1

/* extra #define's to extend Allegro's */
#define MSG_REFRESH MSG_USER
#define D_DRAGGING  D_USER
//#define D_DIRTY    (D_USER << 1)   //Allegro now has this itself

/* basic types */

typedef struct {
   int x,y, x2,y2;
} RECT;

typedef struct {
   int type;
   int x, y;
   int key;
} MSG;

typedef struct {
   MSG msg[MAX_MESSAGES];
   int head, tail;
} MSG_QUEUE;

struct WINDOW;

typedef struct GADGET {
   int (*proc)(MSG *msg, struct WINDOW *window, struct GADGET *d);
   int x,y, w,h;
   int flags;
   int d1, d2;
   void *dp, *dp2, *dp3;
   BITMAP *bmp;
} GADGET;

typedef struct WINDOW {
   struct WINDOW *prev, *next;
   int x,y, w,h;
   MSG_QUEUE msg_queue;
   GADGET *gadgets;
   BITMAP *bmp;
   GADGET *input_focus;
   int flags;
   void (*close_func)(struct WINDOW *window);
} WINDOW;

/* GUI functions */

int GUI_Init(char *datafile);
int GUI_Run(void);
void GUI_Shutdown(void);
void GUI_PostMessage(MSG_QUEUE *queue, MSG *msg);
int GUI_GetMessage(MSG_QUEUE *queue, MSG *msg);
void GUI_PrintLog(int urgent, char *msg, ...) __attribute__ ((format (printf, 2, 3)));
int GUI_MessageBox(int flags, char *msg2, char *msg, ...) __attribute__ ((format (printf, 3, 4)));
void GUI_SetMouseCursor(BITMAP *bmp, int focus_x, int focus_y);

/* Desktop functions */

int GUI_DispatchGlobalMessage(MSG *msg);
int GUI_ProcessWindowMessages(void);
void GUI_SendGadgetMessage(WINDOW *window, GADGET *gadget, MSG *msg);
void GUI_SetStartupFunction(int (*startup)(void), int (*shutdown)(WINDOW *window, GADGET *d));

void GUI_LoadConfig(void);
void GUI_SaveConfig(void);
void GUI_ReadConfigColour(int colour, char *name, int def);
void GUI_SaveConfigColour(int colour, char *name);

void GUI_Refresh(WINDOW *finalwindow);
void GUI_RecursiveRefresh(WINDOW *window, RECT cliprect, WINDOW *finalwindow);
void GUI_CheckWindowForRedraw(WINDOW *window);

WINDOW *GUI_NewWindow(int x, int y, GADGET *gadgets, void (*closefunc)(WINDOW *window));
void GUI_MoveWindow(WINDOW *window, int x, int y);
void GUI_RemoveWindow(WINDOW *window);
void GUI_BringWindowToFront(WINDOW *window);
void GUI_SwitchFocus(WINDOW *newwindow);
void GUI_SwitchInputFocus(WINDOW *window, GADGET *gadget);

WINDOW *GUI_GetWindowAtCoords(int x, int y);
GADGET *GUI_GetGadgetAtCoords(WINDOW *window, int x, int y);
void GUI_DrawRect3D(int fg, BITMAP *bmp, int x1, int y1, int x2, int y2);
void GUI_DrawLine3D(int fg, BITMAP *bmp, int x1, int y1, int x2, int y2);
void GUI_DrawDottedRect(BITMAP *bmp, int x1, int y1, int x2, int y2);

/* Built-in gadget types */
int gui_window_proc(MSG *msg, WINDOW *window, GADGET *d);
int gui_button_proc(MSG *msg, WINDOW *window, GADGET *d);
int gui_closebox_proc(MSG *msg, WINDOW *window, GADGET *d);
int gui_bmp_proc(MSG *msg, WINDOW *window, GADGET *d);
int gui_line_proc(MSG *msg, WINDOW *window, GADGET *d);
int gui_scrollbar_proc(MSG *msg, WINDOW *window, GADGET *d);
int gui_editbox_proc(MSG *msg, WINDOW *window, GADGET *d);
int gui_text_proc(MSG *msg, WINDOW *window, GADGET *d);
int gui_radio_proc(MSG *msg, WINDOW *window, GADGET *d);
int gui_frame_proc(MSG *msg, WINDOW *window, GADGET *d);

/* Built-in filter types */
char gui_hex_filter(char key);
char gui_dec_filter(char key);

/* Log-window stuff */
int GUI_InitLog(void);
void GUI_CloseLog(WINDOW *window);
int gui_log_proc(MSG *msg, WINDOW *window, GADGET *d);

/* GUI variables */
extern int gui_holding_mouse;
extern int gui_mouse_focus_x, gui_mouse_focus_y;
extern BITMAP *gui_mouse_bitmap, *gui_backdrop;
extern MSG_QUEUE global_msg_queue;
extern int gfx_on, gui_display_log_on_exit;
extern DATAFILE *gui_data;

/* Desktop variables */
extern WINDOW *top_window, *bottom_window, *active_window;
extern int (*startup_function)(void);
extern PALETTE gui_palette;
extern int gui_palette_dirty;

/* Log / log-window variables */
extern int log_head, log_tail;
extern GADGET gui_log_dialog[];
extern char log_text[LOG_LINES][300];
extern int log_urgent[LOG_LINES];
extern FILE *log_file;

#endif /* __RM_GUI_INCLUDED__ */
