#include "nz.h"
#include <string.h>

WINDOW *help_window=NULL;
char *help_data;
FONT *bigfont, *verybigfont;
int num_help_links;
HELP_LINK help_links[10];

/* How the hypertext format works:

   To put a damn big header:     *** Put yer text here ***

   To put a sub-header:          Section name
                                 ~~~~~~~~~~~~
   To put a link to a section:   "Section name"

   To make something bold:       *wibble*

   To make a list:               list = foo

   You *cannot* use a " or * anywhere else without causing problems.
*/

int Main_Help(WINDOW *window, GADGET *d)
{
   FILE *help_file;
   int x, y, size;

   if (help_window)
   {
      GUI_SwitchFocus(help_window);
      return D_O_K;
   }

   bigfont = gui_data[GUI_BIG_FONT].dat;
   verybigfont = gui_data[GUI_VERYBIG_FONT].dat;

   GUI_PrintLog(FALSE, "Opening help file (readme.txt)");
   help_file = fopen("readme.txt", "r");
   if (!help_file)
   {
      GUI_PrintLog(TRUE, "Cannot open help file (readme.txt)");
      GUI_MessageBox(MB_OK, NULL, "Cannot open help file (readme.txt)");
      return D_O_K;
   }

   fseek(help_file, 0, SEEK_END);
   size = ftell(help_file);
   fseek(help_file, 0, SEEK_SET);

   /* 'size' = filesize in bytes.
    * NOTE: since we are reading in Unix mode, 'size' will be reported shorter
    * than the real length because CR's are stripped.
    */

   help_data = (char *)malloc(size + 1);
   if (!help_data)
   {
      fclose(help_file);
      GUI_PrintLog(TRUE, "Error loading help file - out of memory");
      GUI_MessageBox(MB_OK, NULL, "Cannot load help file - out of memory");
      return D_O_K;
   }

   size = fread(help_data, 1, size, help_file);
   if (!size)
   {
      free(help_data);
      fclose(help_file);
      GUI_PrintLog(TRUE, "Read error on help file (readme.txt)");
      GUI_MessageBox(MB_OK, NULL, "Disk error while reading help file (readme.txt)");
      return D_O_K;
   }
   fclose(help_file);

   help_data[size] = NULL; /* NULL-terminate the text */

   /* create the help window */
   GUI_PrintLog(FALSE, "Creating help window.");

   x = get_config_int("Help_window", "x_position", 76);
   y = get_config_int("Help_window", "y_position", 26);
   help_window = GUI_NewWindow(x, y, gui_help_dialog, &Help_Shutdown);
   if (!help_window)
      return D_O_K;

   num_help_links = 0;
   return D_O_K;
}

void Help_Shutdown(WINDOW *window)
{
   set_config_int("Help_window", "x_position", window->x);
   set_config_int("Help_window", "y_position", window->y);
   help_window = NULL;
}

int gui_help_proc(MSG *msg, WINDOW *window, GADGET *d)
{
   int n, x, y, section_header_done, big;
   char *text, *p, buf[300];
   HELP_LINK *link;

   switch(msg->type)
   {
   case MSG_START:
      d->dp = help_data;  /* start viewing the first entry */
      return D_O_K;
   case MSG_CLICK:
      link = &help_links[0];
      for (n=0;n<num_help_links;n++,link++)
      {
         if (msg->x >= link->x && msg->y >= link->y
          && msg->x <= link->x2 && msg->y <= link->y2) break;
      }
      if (n == num_help_links) return D_O_K;

      d->dp = Help_FindLink(link->name);
      d->flags |= D_DIRTY;
      return D_O_K;
   case MSG_DRAW:
      text = d->dp;

      text_mode(-1);
      clear_to_color(d->bmp, GUI_WHITE);

      x = 4;
      y = 4;
      section_header_done = FALSE;
      num_help_links = 0;
      while(*text)
      {
         p = text;
         big = FALSE;
         while(1)
         {
            if (*text == 10) /* newline */
            {
               if (*(text+1) == '~') /* section header */
               {
                  /* this line is a section header, stop if needed */
                  if (section_header_done)
                  {
                     /* don't add 'back' link on the 1st page */
                     if (d->dp == help_data) return D_O_K;

                     y += text_height(font);
                     strcpy(buf, "\"Back to the main page\"");
                     y += Help_PrintLine(d->bmp, buf, x, y, FALSE);
                     return D_O_K;
                  }
                  section_header_done = big = TRUE;
               }
               *text = NULL;
               strcpy(&buf[0], p);
               *text = 10;
               text++;
               break;
            }
            if (*text == NULL)
            {
               strcpy(&buf[0], p);
               break;
            }
            text++;
         }

         y += Help_PrintLine(d->bmp, buf, x, y, big);
      }

      y += text_height(font);
      strcpy(buf, "\"Back to the main page\"");
      /* this relies on an undefined link taking the user back
         to the main page */
      y += Help_PrintLine(d->bmp, buf, x, y, FALSE);

      return D_O_K;
   }
   return D_O_K;
}           

int Help_PrintLine(BITMAP *bmp, char *text, int x, int y, int section_header)
{
   char buf[2], *p, *linkname=NULL;
   int w,h, verybig, bold, link, linkstart=0;

   if (*text == '~') return text_height(font);

   verybig = bold = link = FALSE;

   if (*text == '*' && *(text+1) == '*' && *(text+2) == '*')
      verybig = TRUE;

   if (verybig)
   {
      /* remove *** from beginning and end */
      text += 4;
      p = text;
      while(*p++);
      p -= 5;
      *p = NULL;

      /* draw it in a damn big font */
      w = text_length(verybigfont, text);
      x = (bmp->w - w) / 2;
      textout(bmp, verybigfont, text, x, y, GUI_BLACK);
      h = text_height(verybigfont);

      /* underline it */
      rectfill(bmp, x, y+h-4, x+w, y+h-3, GUI_BLACK);
   } else if (section_header) {
      /* draw it in a large font */
      textout(bmp, bigfont, text, x, y, GUI_BLACK);
      w = text_length(bigfont, text);
      h = text_height(bigfont);

      /* underline it */
      hline(bmp, x, y+h-3, x+w, GUI_BLACK);
   } else {
      /* normal paragraph font */
      buf[1] = NULL;
      h = text_height(font);

      while(*text)
      {
         buf[0] = *text++;
         if (buf[0] == '*')
         {
            bold = !bold;
            continue;
         }
         if (buf[0] == '=') /* list */
         {
            x = 200;
            continue;
         }
         if (buf[0] == '\"')
         {
            if (link)
            {
               /* finish this link */
               link = FALSE;
               if (num_help_links < 10)
               {
                  help_links[num_help_links].x = linkstart;
                  help_links[num_help_links].y = y;
                  help_links[num_help_links].x2 = x-1;
                  help_links[num_help_links].y2 = y+h-1;

                  *(text-1) = NULL;
                  strcpy(help_links[num_help_links].name, linkname);
                  num_help_links++;
               }
            } else {
               /* start new link */
               linkstart = x;
               linkname = text;
               link = TRUE;
            }
            continue;
         }

         w = text_length(font, buf);
         if (bold || buf[0] == '*') {
            textout(bmp, font, buf, x, y, GUI_BLACK);
            textout(bmp, font, buf, x+1, y, GUI_BLACK);
            w++;
         } else if (link) {
            textout(bmp, font, buf, x, y, GUI_URGENT_TEXT);
            hline(bmp, x, y+h-gui_font_baseline, x+w-1, GUI_URGENT_TEXT);
         } else {
            textout(bmp, font, buf, x, y, GUI_BLACK);
         }

         x += w;
      }
   }

   return h;
}

char *Help_FindLink(char *name)
{
   char *linestart, *text;

   linestart = text = help_data;

   while(*text)
   {
      if (*text++ == 10) /* LF */
      {
         if (*text == '~')
         {
            if (strncmp(linestart, name, strlen(name)) == 0) return linestart;
         }
         linestart = text;
      }
   }

   return help_data;
}

GADGET gui_help_dialog[] = {
   /* (dialog proc)     (x)          (y)       (w)       (h)           (flags)  (d1) (d2)  (dp) [dp2] */
   { gui_window_proc,   0,           0,        NC_W+500, NC_H+400,     0,       0,   0,    "Help window" },
   { gui_closebox_proc, NC_W+500-12, 2,        9,        9,            D_EXIT,  0,   0,    NULL },
   { gui_help_proc,     NC_X,        NC_Y,     500,      400,          0,       0,   0,    NULL },
   { NULL }
};
